from .request_utils import *

import logging

# Suppress logs below WARNING level

logging.basicConfig(level=logging.WARNING)


def make_request(url, data=None, json_data=None, headers=None, get_post=None, endpoint=None, files=None, status_code=False, retry_after=False, raw_response=False, response_result=None, load_nested_json=True, auth=None, logger=True):
    response = None
    
    # values should contain 'headers', 'url', etc.
    values = get_values_js(url=url, endpoint=endpoint, data=data, headers=headers)
    
    # 1. Handle Files
    if files:
        values['files'] = files
        # If sending files, 'data' must be a dict (form-data), not a JSON string/param
        values['data'] = data 
    # 2. Handle JSON Data
    elif json_data:
        values['json'] = json_data
    # 3. Default behavior for POST/PUT if no files
    elif data and str(get_post).upper() in ['POST', 'PUT']:
        values['json'] = data
    
    if auth:
        values['auth'] = auth
    # Determine method
    get_post = str(get_post or ('POST' if data is None else 'GET')).upper()
    input(values)
    # Use requests.request for a cleaner implementation
    try:
        response = requests.request(method=get_post, **values)
    except Exception as e:
        logging.error(f"Request failed: {e}")
        raise
    return get_response(response, raw_response=raw_response, response_result=response_result, load_nested_json=load_nested_json)


def getRpcData(method=None,params=None,jsonrpc=None,id=None):
    return {
            "jsonrpc": jsonrpc or "2.0",
            "id": 0,
            "method": method,
            "params": params,
        }


def get_request_file(file_path):
    """
    Returns a dictionary structure that 'requests' understands as a 2-tuple.
    Key = The name the server expects (e.g., 'files')
    Value = The open file handle
    """
    try:
        # Open in binary mode
        f = open(file_path, 'rb')
        # This is a dictionary where the value effectively creates a 2-tuple 
        # during the requests internal iteration.
        return {'files': f} 
    except FileNotFoundError:
        logging.error(f"File not found: {file_path}")
        return None


def postRequest(url,
                data=None,
                headers=None,
                endpoint=None,
                request_file_path=None,
                files=None,
                status_code=False,
                retry_after=False,
                raw_response=False,
                response_result=None,
                load_nested_json=True,
                auth=None,
                **kwargs
                ):
    if request_file_path:
        
        files = get_request_file(request_file_path)
    if files and isinstance(files,str):
        files = get_request_file(files)
    data = data or kwargs
    
    return make_request(url,
                        data=data,
                        headers=headers,
                        endpoint=endpoint,
                        get_post='POST',
                        files=files,
                        status_code=status_code,
                        retry_after=retry_after,
                        raw_response=raw_response,
                        response_result=response_result,
                        load_nested_json=load_nested_json,
                        auth=auth
                        )


def getRequest(url, data=None, headers=None, endpoint=None,request_file_path=None,files=None, status_code=False, retry_after=False, raw_response=False, response_result=None, load_nested_json=True,auth=None,**kwargs):
    if request_file_path:
        input(files)
        files = get_request_file(request_file_path)
    if files and isinstance(files,str):
        files = get_request_file(files)
    input(files)
    data = data or kwargs
    return make_request(url, data=data, headers=headers, endpoint=endpoint, get_post='GET', files=files, status_code=status_code, retry_after=retry_after, raw_response=raw_response, response_result=response_result, load_nested_json=load_nested_json,auth=auth)


def makeRequest(url, *args,data=None, headers=None, endpoint=None,get_post=None,request_file_path=None,files=None, status_code=False, retry_after=False, raw_response=False, response_result=None, load_nested_json=True,auth=None,**kwargs):
    if request_file_path:
        files = get_request_file(request_file_path)
    data = data or kwargs
    if not isinstance(data,dict):
        data = {"args":make_list(data or [])}
    data['args'] = make_list(data.get('args') or [])+list(args)
    return make_request(url, data=data, headers=headers, endpoint=endpoint, get_post=get_post, files=files, status_code=status_code, retry_after=retry_after, raw_response=raw_response, response_result=response_result, load_nested_json=load_nested_json,auth=auth)


def getRpcRequest(url, method=None,params=None,jsonrpc=None,id=None,headers=None, endpoint=None, status_code=False, retry_after=False, raw_response=False, response_result=None, load_nested_json=True,auth=None):
    data = getRpcData(method=method,params=params,jsonrpc=jsonrpc,id=id)
    return getRequest(url, data, headers=headers, endpoint=endpoint, status_code=status_code, retry_after=retry_after, raw_response=raw_response, response_result=response_result, load_nested_json=load_nested_json,auth=auth)


def postRpcRequest(url, method=None,params=None,jsonrpc=None,id=None,headers=None, get_post='GET',endpoint=None, status_code=False, retry_after=False, raw_response=False, response_result=None, load_nested_json=True,auth=None):
    data = getRpcData(method=method,params=params,jsonrpc=jsonrpc,id=id)
    return make_request(url=url, data=data, headers=headers, endpoint=endpoint,get_post='POST', status_code=status_code, retry_after=retry_after, raw_response=raw_response, response_result=response_result, load_nested_json=load_nested_json,auth=auth) 



import json,requests,aiohttp,asyncio,os
from flask import jsonify
from abstract_utilities import make_list,eatAll,get_mime_type
def get_headers(is_json=True):
    # Only return JSON headers if we aren't sending files
    if is_json:
        return {'Content-Type': 'application/json'}
    return {} # Let requests handle Content-Type for files

def get_values_js(url=None, data=None, headers=None, endpoint=None, auth=None, files=None):
    if endpoint:
        url = get_url(url, endpoint=endpoint)
    
    values = {'url': url, 'auth': auth}
    
    # Check if we are sending files
    if files:
        values['files'] = files
        values['data'] = data # data becomes form-fields when files are present
        # CRITICAL: Do NOT set application/json headers when sending files
        values['headers'] = headers or get_headers(is_json=False)
    else:
        # Standard JSON logic
        dataKey = 'json' if isinstance(data, dict) else 'data'
        values[dataKey] = data or {}
        values['headers'] = headers or get_headers(is_json=True)
        
    return values
def ensure_json(data):
    if isinstance(data, str):
        try:
            json.loads(data)  # Verify it's valid JSON
            return data
        except ValueError:
            pass  # Not valid JSON, continue to dump it
    return json.dumps(data)

def stripit(string, chars=[]):
    string = string or ''
    for char in make_list(chars):
        string = string.strip(char)
    return string

def make_endpoint(endpoint):
    return eatAll(endpoint, ['/'])

def make_url(url):
    return eatAll(url, ['/'])

def get_url(url, endpoint=None):
    url = eatAll(url, ['/'])
    endpoint =  eatAll(endpoint, ['/'])
    url = f"{url}/{endpoint}"
    return eatAll(url, ['/'])

def get_text_response(response):
    try:
        return response.text
    except Exception as e:
        return None

def load_inner_json(data):
    """Recursively load nested JSON strings within the main JSON response, even if nested within lists."""
    if isinstance(data, str):
        try:
            return load_inner_json(json.loads(data))  # Recursively parse inner JSON strings
        except (ValueError, TypeError):
            return data
    elif isinstance(data, dict):
        return {key: load_inner_json(value) for key, value in data.items()}
    elif isinstance(data, list):
        return [load_inner_json(item) for item in data]
    return data    
def get_status_code(response):
    try:
        return response.status_code
    except Exception as e:
        print(f"Could not get status code: {e}")
        return None
def get_retry_after(response):
    try:
        return response.headers.get("Retry-After")
    except Exception as e:
        print(f"Could not get status code: {e}")
        return None

    
def get_values_js(url=None, data=None, headers=None, endpoint=None, auth=None, files=None):
    if endpoint:
        url = get_url(url, endpoint=endpoint)

    values = {
        "url": url,
    }

    if auth:
        values["auth"] = auth

    if files:
        values["files"] = files
        values["data"] = data or {}

        # Do not set Content-Type manually for multipart.
        # requests will generate multipart/form-data with boundary.
        if headers:
            values["headers"] = {
                k: v for k, v in headers.items()
                if k.lower() != "content-type"
            }

        return values

    values["json"] = data or {}

    if headers:
        values["headers"] = headers
    else:
        values["headers"] = get_headers(is_json=True)

    return values

def get_json_response(response=None, response_result=None, load_nested_json=True,status_code=None,value=None):
    if (response==None and response_result==None) and (status_code or value):
        return jsonify({'result':value}),status_code
    response_result = response_result or 'result'
    
    try:
        try:
            response_json = response.json()
        except Exception as e:
            response_json = response
        if isinstance(response_json,dict):
            response_json = response_json.get(response_result, response_json)

        if response_json is not None:
            return response_json
    except Exception as e:
        print(e)
        return response_result
    return response_result
def get_response(response, response_result=None, raw_response=False, load_nested_json=True):
    if raw_response:
        return response
    json_response = get_json_response(response, response_result=response_result, load_nested_json=load_nested_json)
    if json_response is not None:
        return json_response
    text_response = get_text_response(response)
    if text_response:
        return text_response
    return response  # Return raw content as a last resort
def getRpcData(method=None,params=None,jsonrpc=None,id=None):
    return {
            "jsonrpc": jsonrpc or "2.0",
            "id": id or 1,
            "method": method,
            "params": params or [],
        }
def get_request_file(file_path, field_name="files"):
    if not os.path.exists(file_path):
        raise FileNotFoundError(f"File not found: {file_path}")

    filename = os.path.basename(file_path)
    mime_type = get_mime_type(file_path) or "application/octet-stream"

    file_handle = open(file_path, "rb")

    return {
        field_name: (
            filename,
            file_handle,
            mime_type,
        )
    }
